@@ -1113,7 +1113,13 @@ pub fn push_down_index(node: &Index) -> Option<CircuitRc> {
                     })
                     .collect(),
             );
-            if index_non_axis.is_identity(&inner.info().shape) {
+            let sections = &inner.get_sections();
+            let old_index_on_concat_axis = node.index.0[inner.axis].clone();
+            let concat_axis_indices_maybe =
+                old_index_on_concat_axis.split_sections_for_concat(sections);
+            if index_non_axis.is_identity(&inner.info().shape)
+                && concat_axis_indices_maybe.is_none()
+            {
                 return None;
             }
             // todo: drop concat parts that get indexed out
@@ -1122,22 +1128,52 @@ pub fn push_down_index(node: &Index) -> Option<CircuitRc> {
                     .iter()
                     .filter(|idx| matches!(idx, TensorAxisIndex::Single(_)))
                     .count();
-
-            let new_concat = Concat::nrc(
-                inner
-                    .nodes
-                    .iter()
-                    .map(|operand| Index::nrc(operand.clone(), index_non_axis.clone(), None))
-                    .collect(),
-                new_axis,
-                inner.name_cloned(),
-            );
-            let final_index = TensorIndex::new_single(
-                node.index.0[inner.axis].clone(),
-                new_axis,
-                new_concat.info().rank(),
-            );
-            Some(Index::nrc(new_concat, final_index, None))
+            if let Some(concat_indices) = concat_axis_indices_maybe {
+                if matches!(old_index_on_concat_axis, TensorAxisIndex::Single(_)) {
+                    let mut index = index_non_axis.clone();
+                    for (i, ax) in concat_indices.iter().enumerate() {
+                        if let Some(z) = ax {
+                            index.0[inner.axis] = z.clone();
+                            return Some(Index::nrc(
+                                inner.nodes[i].clone(),
+                                index,
+                                node.name_cloned(),
+                            ));
+                        }
+                    }
+                    None
+                } else {
+                    Some(Concat::nrc(
+                        zip(inner.nodes.iter(), concat_indices)
+                            .filter_map(|(child, idx)| {
+                                idx.map(|idx| {
+                                    let mut index = index_non_axis.clone();
+                                    index.0[inner.axis] = idx.clone();
+                                    Index::nrc(child.clone(), index, node.name_cloned())
+                                })
+                            })
+                            .collect(),
+                        new_axis,
+                        inner.name_cloned(),
+                    ))
+                }
+            } else {
+                let new_concat = Concat::nrc(
+                    inner
+                        .nodes
+                        .iter()
+                        .map(|operand| Index::nrc(operand.clone(), index_non_axis.clone(), None))
+                        .collect(),
+                    new_axis,
+                    inner.name_cloned(),
+                );
+                let final_index = TensorIndex::new_single(
+                    node.index.0[inner.axis].clone(),
+                    new_axis,
+                    new_concat.info().rank(),
+                );
+                Some(Index::nrc(new_concat, final_index, None))
+            }
         }
         Circuit::GeneralFunction(inner) => {
             if !inner.spec.is_batchable()
