@@ -1,23 +1,28 @@
-use crate::{all_imports::Shape, circuit::repr::RustRepr, hashmaps::FxHashMap as HashMap};
+use crate::{
+    all_imports::Shape,
+    circuit::{circuit_utils::has_leaf_symbols, repr::RustRepr},
+    hashmaps::FxHashMap as HashMap,
+};
 
 use num_bigint::BigUint;
 
-use crate::{circuit::print::rust_expression_notation_circuit, pyo3_prelude::*};
-
 use super::{
     canonicalize::deep_canonicalize,
     compiler_heuristics::deep_maybe_distribute,
     compiler_strip::strip_names,
     debugging::{create_circuit_test, randn_ablate_great_grandchildren},
+    deep_map_op_context,
     deep_rewrite::deep_heuristic_nest_adds,
     deep_rewrite::{compiler_simp, deep_optimize_einsums, deep_push_down_index},
     flat_concat, get_compatible_dtype,
     scheduled_execution::{
         circuit_to_schedule, circuit_to_schedule_naive_toposort, Schedule, SchedulingError,
     },
-    CircuitNode, CircuitRc, HashBytes,
+    CircuitNode, CircuitRc, HashBytes, ModuleNode, ModuleNodeSpec,
 };
+use crate::circuit::CircResult;
 use crate::opt_einsum::EinsumSpec;
+use crate::{circuit::print::rust_expression_notation_circuit, pyo3_prelude::*};
 use crate::{py_types::Tensor, timed, util::apply_fn_until_same};
 
 // a lot of the function boundries in this are there because it would be too hard to optimize things jointly
@@ -165,6 +170,11 @@ pub struct OptimizationCache {
     pub flops: HashMap<HashBytes, BigUint>,
     pub sum_of_node_sizes: HashMap<HashBytes, BigUint>,
     pub canonicalized: HashMap<HashBytes, Option<CircuitRc>>,
+    pub module_specs_optimized_same_settings: HashMap<ModuleNodeSpec, ModuleNodeSpec>,
+    pub module_specs_scheduled_same_settings: HashMap<ModuleNodeSpec, Schedule>,
+    pub module_specs_expanded_shape: HashMap<(ModuleNodeSpec, Vec<Shape>), CircResult>,
+    pub module_specs_expanded: HashMap<(ModuleNodeSpec, Vec<HashBytes>), CircResult>,
+    pub stripped_names: HashMap<HashBytes, Option<CircuitRc>>,
     pub times_distributed: usize,
     pub simplification_log: Vec<&'static str>,
     pub max_tensor_elements: usize,
@@ -173,18 +183,24 @@ pub struct OptimizationCache {
     pub slow_einsum_log: Vec<EinsumSpec>,
 }
 
-#[derive(Debug, Clone, Default)]
+/// Note: you can't change the settings once anything's been cached
+/// bc cache assumes same settings on everything
+#[pyclass(unsendable)]
+#[derive(Debug, Clone, Default, PyClassDeriv)]
 pub struct OptimizationContext {
     pub cache: OptimizationCache,
     pub settings: OptimizationSettings,
 }
+#[pymethods]
 impl OptimizationContext {
+    #[staticmethod]
     pub fn new_settings(settings: OptimizationSettings) -> Self {
         Self {
             cache: Default::default(),
             settings,
         }
     }
+    #[staticmethod]
     pub fn new_settings_circuit(settings: OptimizationSettings, circuit: CircuitRc) -> Self {
         let mut result = Self {
             cache: Default::default(),
@@ -219,7 +235,8 @@ pub fn optimize_circuit_py(circuit: CircuitRc, settings: OptimizationSettings) -
 
 pub fn optimize_circuit(circuit: CircuitRc, context: &mut OptimizationContext) -> CircuitRc {
     let print_timings = context.settings.verbose >= 2;
-    let circuit = timed!(strip_names(circuit), 10, print_timings);
+    let circuit = timed!(strip_names(circuit, context), 10, print_timings);
+    let circuit = timed!(deep_optimize_module_specs(circuit, context));
     let circuit = timed!(deep_canonicalize(circuit, context), 10, print_timings);
     if context.settings.verbose >= 4 {
         println!("Original Circuit");
@@ -317,6 +334,9 @@ pub fn optimize_and_evaluate(
     circuit: CircuitRc,
     settings: OptimizationSettings,
 ) -> Result<Tensor, SchedulingError> {
+    if has_leaf_symbols(circuit.clone(), &Default::default()) {
+        return Err(SchedulingError::LeafSymbols {});
+    }
     let schedule = optimize_to_schedule(circuit, settings)?;
 
     Ok(schedule.evaluate(settings))
@@ -352,3 +372,47 @@ pub fn optimize_to_schedule_many(
     );
     Ok(schedule)
 }
+
+pub fn optimize_module_spec(
+    spec: ModuleNodeSpec,
+    context: &mut OptimizationContext,
+) -> ModuleNodeSpec {
+    context
+        .cache
+        .module_specs_optimized_same_settings
+        .get(&spec)
+        .cloned()
+        .unwrap_or_else(|| {
+            let result = ModuleNodeSpec {
+                input_specs: spec.input_specs.clone(),
+                name: spec.name.clone(),
+                spec_circuit: optimize_circuit(spec.spec_circuit.clone(), context),
+            };
+            context
+                .cache
+                .module_specs_optimized_same_settings
+                .insert(spec, result.clone());
+            result
+        })
+}
+
+pub fn deep_optimize_module_specs(
+    circuit: CircuitRc,
+    context: &mut OptimizationContext,
+) -> CircuitRc {
+    deep_map_op_context(
+        &circuit,
+        &|circuit, context| {
+            circuit.as_module_node().map(|mn| {
+                ModuleNode::nrc(
+                    mn.nodes.clone(),
+                    optimize_module_spec(mn.spec.clone(), context),
+                    mn.name_cloned(),
+                )
+            })
+        },
+        context,
+        &mut Default::default(),
+    )
+    .unwrap_or(circuit.clone())
+}
