@@ -297,12 +297,20 @@ impl Einsum {
 impl CircuitNodeAutoName for Einsum {
     fn auto_name(&self, name: Option<String>) -> Option<String> {
         name.or_else(|| {
-            if self.input_circuits().all(|x| x.name().is_none()) {
+            if self.input_circuits().any(|x| x.name().is_none()) {
                 None
             } else {
                 Some(
                     self.input_circuits()
-                        .filter_map(|x| x.name().map(|y| y.to_owned()))
+                        .filter_map(|x| {
+                            x.name().map(|y| {
+                                if y.len() > 100 {
+                                    "...".to_owned()
+                                } else {
+                                    y.to_owned()
+                                }
+                            })
+                        })
                         .collect::<Vec<String>>()
                         .join(" * "),
                 )
@@ -602,12 +610,20 @@ impl CircuitNode for Add {
 impl CircuitNodeAutoName for Add {
     fn auto_name(&self, name: Option<String>) -> Option<String> {
         name.or_else(|| {
-            if self.children().all(|x| x.name().is_none()) {
+            if self.children().any(|x| x.name().is_none()) {
                 None
             } else {
                 Some(
                     self.children()
-                        .filter_map(|x| x.name().map(|y| y.to_owned()))
+                        .filter_map(|x| {
+                            x.name().map(|y| {
+                                if y.len() > 100 {
+                                    "...".to_owned()
+                                } else {
+                                    y.to_owned()
+                                }
+                            })
+                        })
                         .collect::<Vec<String>>()
                         .join(" + "),
                 )
@@ -1005,6 +1021,33 @@ impl Index {
 
         Ok(out.into_init())
     }
+
+    pub fn slice_edges_to_none(&self) -> Self {
+        Index::try_new(
+            self.node.clone(),
+            TensorIndex(
+                zip(&self.index.0, &self.node.info().shape)
+                    .map(|(ax, l)| match ax {
+                        TensorAxisIndex::Slice(slice) => TensorAxisIndex::Slice(Slice {
+                            start: if slice.start_u(*l) == 0 {
+                                None
+                            } else {
+                                slice.start
+                            },
+                            stop: if slice.stop_u(*l) == *l {
+                                None
+                            } else {
+                                slice.stop
+                            },
+                        }),
+                        _ => ax.clone(),
+                    })
+                    .collect(),
+            ),
+            self.name_cloned(),
+        )
+        .unwrap()
+    }
 }
 
 #[pyclass(unsendable, extends=PyCircuitBase)]
@@ -1123,12 +1166,20 @@ impl CircuitNode for Concat {
 impl CircuitNodeAutoName for Concat {
     fn auto_name(&self, name: Option<String>) -> Option<String> {
         name.or_else(|| {
-            if self.children().all(|x| x.name().is_none()) {
+            if self.children().any(|x| x.name().is_none()) {
                 None
             } else {
                 Some(
                     self.children()
-                        .filter_map(|x| x.name().map(|y| y.to_owned()))
+                        .filter_map(|x| {
+                            x.name().map(|y| {
+                                if y.len() > 100 {
+                                    "...".to_owned()
+                                } else {
+                                    y.to_owned()
+                                }
+                            })
+                        })
                         .collect::<Vec<String>>()
                         .join(" ++ "),
                 )
