@@ -306,7 +306,7 @@ impl CircuitNode for GeneralFunction {
 impl CircuitNodeAutoName for GeneralFunction {
     fn auto_name(&self, name: Option<String>) -> Option<String> {
         name.or_else(|| {
-            if self.children().all(|x| x.name().is_none()) {
+            if self.children().any(|x| x.name().is_none()) {
                 None
             } else {
                 Some(
@@ -314,7 +314,15 @@ impl CircuitNodeAutoName for GeneralFunction {
                         + " "
                         + &self
                             .children()
-                            .filter_map(|x| x.name().map(|y| y.to_owned()))
+                            .filter_map(|x| {
+                                x.name().map(|y| {
+                                    if y.len() > 100 {
+                                        "...".to_owned()
+                                    } else {
+                                        y.to_owned()
+                                    }
+                                })
+                            })
                             .collect::<Vec<String>>()
                             .join(" , "),
                 )
