@@ -530,35 +530,6 @@ define_circuit!(
 );
 
 /// Define adhoc unions of different circuit types
-///
-/// # Example
-///
-/// ```
-/// # use rust_circuit::{
-/// #     circuit::{ArrayConstant, ScalarConstant, Symbol, CircuitNodeUnion, CircuitNode, Circuit},
-/// #     define_circuit_union, util::AsOp, sv
-/// # };
-/// # use uuid::Uuid;
-///
-/// define_circuit_union!(SomeConst {
-///     ScalarConstant,
-///     Symbol,
-///     ArrayConstant,
-/// });
-///
-/// let values = [
-///     SomeConst::ScalarConstant(ScalarConstant::new(0.2, sv![3], Some("first".to_owned()))),
-///     SomeConst::Symbol(Symbol::new(sv![3], Uuid::new_v4(), Some("second".to_owned()))),
-/// ];
-///
-/// assert_eq!(values[0].name(), Some("first"));
-/// assert_eq!(values[1].name(), Some("second"));
-/// assert_eq!(values[0].variant_string(), "ScalarConstant".to_owned());
-/// assert_eq!(values[1].variant_string(), "Symbol".to_owned());
-///
-/// // we can convert to full circuit also
-/// let values_circuit : Vec<Circuit> = values.iter().cloned().map(Into::into).collect();
-/// ```
 #[macro_export]
 macro_rules! define_circuit_union {
     [$name:ident {$($x:ident),+ $(,)?}] => {
@@ -573,8 +544,8 @@ macro_rules! define_circuit_union {
                 }
             }
         }
-        impl ::std::convert::From<$crate::circuit::Circuit> for Option<$name> {
-            fn from(item: $crate::circuit::Circuit) -> Option<$name> {
+        impl ::std::convert::From<$crate::circuit::Circuit> for ::std::option::Option<$name> {
+            fn from(item: $crate::circuit::Circuit) -> ::std::option::Option<$name> {
                 match item {
                     $(
                         $crate::circuit::Circuit::$x(node) => Some(node.into()),
@@ -585,7 +556,7 @@ macro_rules! define_circuit_union {
         }
         impl $name{
             pub fn matches(circuit:&$crate::circuit::Circuit)->bool{
-                let op:Option<$name>=circuit.clone().into();
+                let op: ::std::option::Option<$name>=circuit.clone().into();
                 op.is_some()
             }
         }
