@@ -1,13 +1,20 @@
 use crate::all_imports::{TensorAxisIndex, TensorIndex};
 use crate::circuit::deep_map_fallible_pre_new_children;
-use crate::hashmaps::FxHashMap as HashMap;
+use crate::hashmaps::{AHashSet as HashSet, FxHashMap as HashMap};
 
+use crate::util::dict_to_list;
 use crate::{pycall, pyo3_prelude::*, sv};
+use once_cell::sync::{Lazy, OnceCell};
+use regex::Regex;
 use std::iter::zip;
+use std::sync::Mutex;
 
+use super::circuit_manipulation::replace_nodes;
+use super::circuit_utils::has_leaf_symbols;
 use super::{
-    deep_map_unwrap_preorder, prelude::*, Add, CachedCircuitInfo, CircResult, Concat,
-    GeneralFunction, HashBytes, Index, Rearrange, Scatter,
+    deep_map_op_context, deep_map_op_context_preorder_stoppable, deep_map_op_pre_new_children,
+    deep_map_unwrap, deep_map_unwrap_preorder, prelude::*, visit_circuit, Add, CachedCircuitInfo,
+    CircResult, Concat, GeneralFunction, HashBytes, Index, Rearrange, Scatter,
 };
 
 use crate::{
@@ -16,10 +23,11 @@ use crate::{
     tensor_util::{Shape, TorchDeviceDtype},
 };
 
+use cached::lazy_static::lazy_static;
 use macro_rules_attribute::apply;
 use pyo3::types::PyDict;
 
-use uuid::uuid;
+use uuid::{uuid, Uuid};
 
 #[pyfunction]
 #[pyo3(name = "expand_node")]
@@ -53,9 +61,9 @@ pub fn expand_node(circuit: &Circuit, inputs: &Vec<CircuitRc>) -> CircResult {
                 .iter()
                 .cloned()
                 .collect();
-            let mut new_spec = node
+            let new_spec = node
                 .spec
-                .conform_to_input_shape(&input_shape_non_batch, false)
+                .conform_to_input_shape(&input_shape_non_batch, true)
                 .map_err(|e| CircuitConstructionError::RearrangeWrongInputShape {
                     spec: node.spec.clone(),
                     shape: input_shape_non_batch.clone(),
@@ -93,12 +101,16 @@ pub fn expand_node(circuit: &Circuit, inputs: &Vec<CircuitRc>) -> CircResult {
         }
         Circuit::Scatter(node) => {
             // for now non-batch non-identity dims can't change
-            if node.node.info().shape[..] != inputs[0].info().shape[batch_ranks[0]..] {
-                return Err(CircuitConstructionError::ExpandingFixedIndex {
-                    index: node.index.clone(),
-                    old_shape: node.node.info().shape.clone(),
-                    new_shape: inputs[0].info().shape.clone(),
-                });
+            for i in 0..node.node.info().rank() {
+                if node.node.info().shape[i] != inputs[0].info().shape[i + batch_ranks[0]]
+                    && node.index.0[i] != TensorAxisIndex::IDENT
+                {
+                    return Err(CircuitConstructionError::ExpandingFixedIndex {
+                        index: node.index.clone(),
+                        old_shape: node.node.info().shape.clone(),
+                        new_shape: inputs[0].info().shape.clone(),
+                    });
+                }
             }
             Ok(Scatter::nrc(
                 inputs[0].clone(),
@@ -123,6 +135,7 @@ pub fn expand_node(circuit: &Circuit, inputs: &Vec<CircuitRc>) -> CircResult {
                         .iter()
                         .map(|x| x.iter().cloned().collect())
                         .collect(),
+                    circuit: circuit.clone().rc(),
                 });
             }
             if !zip(&node.nodes, zip(inputs, &batch_ranks)).all(|(old, (new, br))| {
@@ -157,6 +170,7 @@ pub fn expand_node(circuit: &Circuit, inputs: &Vec<CircuitRc>) -> CircResult {
                                     .iter()
                                     .map(|x| x.iter().cloned().collect())
                                     .collect(),
+                                circuit: circuit.clone().rc(),
                             });
                         }
                     } else {
@@ -190,9 +204,19 @@ pub fn expand_node(circuit: &Circuit, inputs: &Vec<CircuitRc>) -> CircResult {
             )
             .map(|x| x.rc())
         }
-        _ => Err(CircuitConstructionError::UnknownGeneralFunction {
-            spec_name: "hi".to_owned(),
-        }),
+        Circuit::ModuleNode(node) => {
+            ModuleNode::try_new(inputs.clone(), node.spec.clone(), node.name_cloned())
+                .map(|z| z.rc())
+        }
+        _ => {
+            if inputs[..] == circuit.children().collect::<Vec<_>>()[..] {
+                Ok(circuit.clone().rc())
+            } else {
+                Err(CircuitConstructionError::ExpandNodeUnknownVariant {
+                    variant: circuit.variant_string().to_owned(),
+                })
+            }
+        }
     }
 }
 
@@ -241,7 +265,8 @@ pub struct ModuleNodeSpec {
 }
 
 impl ModuleNodeSpec {
-    pub fn expand(&self, nodes: &Vec<CircuitRc>) -> CircResult {
+    pub const EXPAND_PLACEHOLDER_UUID: Uuid = uuid!("741ba404-eec3-4ac9-b6ce-062e903fb033");
+    pub fn expand_raw(&self, nodes: &Vec<CircuitRc>) -> CircResult {
         if self.input_specs.len() != nodes.len() {
             return Err(CircuitConstructionError::ModuleNodeWrongNumberChildren {
                 expected: self.input_specs.len(),
@@ -251,21 +276,48 @@ impl ModuleNodeSpec {
         for (spec, node) in zip(self.input_specs.iter(), nodes) {
             if node.info().rank() < spec.symbol.info().rank() {
                 // todo error messages
+                println!(
+                    "node shapes {:?}",
+                    nodes
+                        .iter()
+                        .map(|x| x.info().shape.clone())
+                        .collect::<Vec<_>>()
+                );
+                self.spec_circuit.compiler_print();
                 return Err(CircuitConstructionError::InconsistentBatches {
-                    batch_shapes: vec![],
+                    batch_shapes: vec![spec.symbol.info().shape.clone(), node.info().shape.clone()],
+                    circuit: self.spec_circuit.clone(),
                 });
             }
             if !spec.batchable && node.info().rank() > spec.symbol.info().rank() {
+                println!(
+                    "node shapes {:?}",
+                    nodes
+                        .iter()
+                        .map(|x| x.info().shape.clone())
+                        .collect::<Vec<_>>()
+                );
+                self.spec_circuit.compiler_print();
                 return Err(CircuitConstructionError::InconsistentBatches {
-                    batch_shapes: vec![],
+                    batch_shapes: vec![spec.symbol.info().shape.clone(), node.info().shape.clone()],
+                    circuit: self.spec_circuit.clone(),
                 });
             }
             if !spec.expandable
                 && node.info().shape[node.info().rank() - spec.symbol.info().rank()..]
                     != spec.symbol.info().shape[..]
             {
+                println!(
+                    "node shapes {:?}",
+                    nodes
+                        .iter()
+                        .map(|x| x.info().shape.clone())
+                        .collect::<Vec<_>>()
+                );
+                self.spec_circuit.compiler_print();
                 return Err(CircuitConstructionError::InconsistentBatches {
-                    batch_shapes: vec![],
+                    batch_shapes: vec![spec.symbol.info().shape.clone(), node.info().shape.clone()],
+                    circuit: self.spec_circuit.clone(),
                 });
             }
         }
@@ -282,6 +334,66 @@ impl ModuleNodeSpec {
         })
     }
 
+    pub fn expand_shape(&self, shapes: &Vec<Shape>) -> CircResult {
+        if let Some(result) = MODULE_EXPANSIONS_SHAPE
+            .with(|cache| cache.borrow().get(&(self.clone(), shapes.clone())).cloned())
+        {
+            return result;
+        }
+        let symbols = shapes
+            .iter()
+            .enumerate()
+            .map(|(i, s)| {
+                Symbol::nrc(
+                    s.clone(),
+                    ModuleNodeSpec::EXPAND_PLACEHOLDER_UUID,
+                    Some(format!("{}_{:?}", i, s)),
+                )
+            })
+            .collect();
+        let result = self.expand_raw(&symbols);
+        MODULE_EXPANSIONS_SHAPE.with(|cache| {
+            cache
+                .borrow_mut()
+                .insert((self.clone(), shapes.clone()), result.clone())
+        });
+        result
+    }
+
+    pub fn expand(&self, nodes: &Vec<CircuitRc>) -> CircResult {
+        let key: (ModuleNodeSpec, Vec<HashBytes>) =
+            (self.clone(), nodes.iter().map(|x| x.info().hash).collect());
+        MODULE_EXPANSIONS
+            .with(|cache| cache.borrow().get(&key).cloned())
+            .unwrap_or_else(|| {
+                let result = (|| {
+                    let shapes = nodes.iter().map(|x| x.info().shape.clone()).collect();
+                    let expanded_shape = self.expand_shape(&shapes)?;
+                    Ok(replace_nodes(
+                        expanded_shape,
+                        &nodes
+                            .iter()
+                            .enumerate()
+                            .map(|(i, n)| {
+                                (
+                                    Symbol::new(
+                                        n.info().shape.clone(),
+                                        ModuleNodeSpec::EXPAND_PLACEHOLDER_UUID,
+                                        Some(format!("{}_{:?}", i, n.info().shape)),
+                                    )
+                                    .info()
+                                    .hash,
+                                    n.clone(),
+                                )
+                            })
+                            .collect(),
+                    ))
+                })();
+                MODULE_EXPANSIONS.with(|cache| cache.borrow_mut().insert(key, result.clone()));
+                result
+            })
+    }
+
     pub fn compute_hash(&self) -> HashBytes {
         let mut hasher = blake3::Hasher::new();
         hasher.update(&self.spec_circuit.info().hash);
@@ -306,6 +418,125 @@ impl ModuleNodeSpec {
             name,
         }
     }
+
+    #[staticmethod]
+    pub fn new_auto(
+        circuit: CircuitRc,
+        name: Option<String>,
+    ) -> Result<Self, CircuitConstructionError> {
+        let mut input_specs_dict: HashMap<usize, ModuleNodeArgSpec> = HashMap::new();
+        let circuit = deep_map_op_context(
+            &circuit,
+            &|sub: &Circuit, input_specs_dict: &mut HashMap<usize, ModuleNodeArgSpec>| {
+                if let Some(sym) = sub.as_symbol() {
+                    static RE_NUM_BEGIN: Lazy<Regex> =
+                        Lazy::new(|| Regex::new(r"#(\d+) (.*)").unwrap());
+                    let captures = RE_NUM_BEGIN.captures(sym.name().unwrap()).unwrap();
+                    let num = captures.get(1).unwrap().as_str().parse::<usize>().unwrap();
+                    let name = captures.get(2).unwrap().as_str().to_owned();
+                    let newsym = Symbol::new(sym.info().shape.clone(), sym.uuid, Some(name));
+                    input_specs_dict.insert(
+                        num,
+                        ModuleNodeArgSpec {
+                            symbol: newsym.clone(),
+                            batchable: true,
+                            expandable: true,
+                        },
+                    );
+                    return Some(newsym.rc());
+                }
+                None
+            },
+            &mut input_specs_dict,
+            &mut Default::default(),
+        )
+        .unwrap_or(circuit);
+        let mut input_specs =
+            vec![input_specs_dict.values().last().unwrap().clone(); input_specs_dict.len()];
+        for (k, v) in input_specs_dict {
+            input_specs[k] = v;
+        }
+        Ok(Self {
+            spec_circuit: circuit,
+            input_specs,
+            name,
+        })
+    }
+
+    #[staticmethod]
+    #[args(require_all_inputs = "false")]
+    pub fn new_extract(
+        circuit: CircuitRc,
+        input_specs: Vec<(CircuitRc, ModuleNodeArgSpec)>,
+        name: Option<String>,
+        require_all_inputs: bool,
+    ) -> Option<Self> {
+        let mut new_input_specs: Vec<Option<ModuleNodeArgSpec>> = vec![None; input_specs.len()];
+        let spec_circuit = deep_map_op_context_preorder_stoppable(
+            &circuit,
+            &|circuit,
+              c: &mut (
+                &mut Vec<Option<ModuleNodeArgSpec>>,
+                &Vec<(CircuitRc, ModuleNodeArgSpec)>,
+            )| {
+                let (real_input_specs, proposed_input_specs) = c;
+                if let Some(i) = proposed_input_specs
+                    .iter()
+                    .position(|x| x.0.info().hash == circuit.info().hash)
+                {
+                    let mut argspec = proposed_input_specs[i].1.clone();
+                    argspec.symbol = Symbol::new(
+                        circuit.info().shape.clone(),
+                        argspec.symbol.uuid,
+                        argspec.symbol.name_cloned().or(circuit.name_cloned()),
+                    );
+                    real_input_specs[i] = Some(argspec);
+                    return (
+                        Some(real_input_specs[i].as_ref().unwrap().symbol.clone().rc()),
+                        true,
+                    );
+                }
+                (None, false)
+            },
+            &mut (&mut new_input_specs, &input_specs),
+            &mut Default::default(),
+        )?;
+        let new_input_specs: Vec<ModuleNodeArgSpec> = if require_all_inputs {
+            new_input_specs.into_iter().collect::<Option<Vec<_>>>()?
+        } else {
+            new_input_specs
+                .into_iter()
+                .filter(|z| z.is_some())
+                .collect::<Option<Vec<_>>>()
+                .unwrap()
+        };
+        Some(Self {
+            spec_circuit,
+            input_specs: new_input_specs,
+            name,
+        })
+    }
+
+    pub fn resize(&self, shapes: Vec<Shape>) -> Result<Self, CircuitConstructionError> {
+        let input_specs: Vec<ModuleNodeArgSpec> = zip(&self.input_specs, shapes)
+            .map(|(spec, shape)| ModuleNodeArgSpec {
+                symbol: Symbol::new(shape, spec.symbol.uuid.clone(), spec.symbol.name_cloned()),
+                batchable: spec.batchable,
+                expandable: spec.expandable,
+            })
+            .collect();
+        let spec_circuit = replace_expand_bottom_up(self.spec_circuit.clone(), |c| {
+            self.input_specs
+                .iter()
+                .position(|x| x.symbol.info().hash == c.info().hash)
+                .map(|p| input_specs[p].symbol.clone().rc())
+        })?;
+        Ok(Self {
+            spec_circuit,
+            input_specs,
+            name: self.name.clone(),
+        })
+    }
 }
 
 #[pyclass(unsendable)]
@@ -337,8 +568,6 @@ pub struct ModuleNode {
     pub nodes: Vec<CircuitRc>,
     #[pyo3(get)]
     pub spec: ModuleNodeSpec,
-    #[pyo3(get)]
-    pub cached_full_circuit: CircuitRc,
     info: CachedCircuitInfo,
     #[pyo3(get)]
     name: Option<String>,
@@ -351,18 +580,22 @@ impl ModuleNode {
         spec: ModuleNodeSpec,
         name: Option<String>,
     ) -> (Result<Self, CircuitConstructionError>) {
-        let cached_full_circuit = spec.expand(&nodes).map_err(|e| {
-            CircuitConstructionError::ModuleNodeExpansionError { error: Box::new(e) }
-        })?;
+        let shape = spec
+            .expand_shape(&nodes.iter().map(|x| x.info().shape.clone()).collect())
+            .map_err(|e| CircuitConstructionError::ModuleNodeExpansionError { error: Box::new(e) })?
+            .info()
+            .shape
+            .clone();
         let mut out = Self {
             nodes,
             spec,
-            cached_full_circuit,
             name: name.clone(),
             info: Default::default(),
         };
+        out.info.shape = shape;
         out.name = out.auto_name(name);
-        out.init_info()
+        let out = out.init_info();
+        out
     }
 
     pub fn new_kwargs(
@@ -397,7 +630,7 @@ impl CircuitNode for ModuleNode {
     circuit_node_auto_impl!("6825f723-f178-4dab-b568-cd85eb6d2bf3");
 
     fn compute_shape(&self) -> Shape {
-        self.cached_full_circuit.info().shape.clone()
+        self.info().shape.clone()
     }
 
     fn compute_hash(&self) -> blake3::Hasher {
@@ -449,7 +682,7 @@ impl CircuitNode for ModuleNode {
 impl CircuitNodeAutoName for ModuleNode {
     fn auto_name(&self, name: Option<String>) -> Option<String> {
         name.or_else(|| {
-            if self.children().all(|x| x.name().is_none()) || self.spec.name.is_none() {
+            if self.children().any(|x| x.name().is_none()) || self.spec.name.is_none() {
                 None
             } else {
                 Some(
@@ -457,7 +690,15 @@ impl CircuitNodeAutoName for ModuleNode {
                         + " "
                         + &self
                             .children()
-                            .filter_map(|x| x.name().map(|y| y.to_owned()))
+                            .filter_map(|x| {
+                                x.name().map(|y| {
+                                    if y.len() > 100 {
+                                        "...".to_owned()
+                                    } else {
+                                        y.to_owned()
+                                    }
+                                })
+                            })
                             .collect::<Vec<String>>()
                             .join(" , "),
                 )
@@ -488,12 +729,26 @@ impl ModuleNode {
     ) -> Result<Self, CircuitConstructionError> {
         Self::try_new(nodes, spec, name)
     }
+
+    pub fn expand(&self) -> CircuitRc {
+        self.spec
+            .expand(&self.nodes)
+            .map_err(|e| CircuitConstructionError::ModuleNodeExpansionError { error: Box::new(e) })
+            .unwrap()
+    }
 }
 
 #[pyfunction]
 pub fn inline_all_modules(circuit: CircuitRc) -> CircuitRc {
-    deep_map_unwrap_preorder(&circuit, |c| match c {
-        Circuit::ModuleNode(mn) => mn.cached_full_circuit.clone(),
+    deep_map_unwrap(&circuit, |c| match c {
+        Circuit::ModuleNode(mn) => inline_all_modules(mn.expand().clone()),
         _ => c.clone().rc(),
     })
 }
+use std::cell::RefCell;
+thread_local! {
+    static MODULE_EXPANSIONS: RefCell<HashMap<(ModuleNodeSpec, Vec<HashBytes>), CircResult>> =
+        RefCell::new(HashMap::new());
+    static MODULE_EXPANSIONS_SHAPE: RefCell<HashMap<(ModuleNodeSpec, Vec<Shape>), CircResult>> =
+        RefCell::new(HashMap::new());
+}
