@@ -114,13 +114,11 @@ pub fn repr_circuit_deep_compiler(
                 _ => "".to_owned(),
             }
         });
-        if let Some(real_named_axes)= &circ.info().named_axes && real_named_axes.iter().any(|x| x.is_some()) && !bijection {
+        if !circ.info().named_axes.is_empty() {
             result.push_str(&format!(
                 " NA[{}]",
-                circ.info()
-                    .named_axes.as_ref().unwrap()
-                    .iter()
-                    .map(|x| match x {
+                (0..circ.info().rank())
+                    .map(|x| match circ.info().named_axes.get(&(x as u8)) {
                         None => "".to_owned(),
                         Some(s) => s.clone(),
                     })
