@@ -1,7 +1,4 @@
-use std::{
-    fs::File,
-    io::{Seek, Write},
-};
+use std::{fs::File, io::Write};
 
 use itertools::Itertools;
 use smallvec::Array;
