@@ -171,7 +171,6 @@ pub fn add_pull_scatter(add: &Add) -> Option<Scatter> {
             };
         }
     }
-    dbg!(&slices);
     if zip(&slices, &add.info().shape).all(|(s, l)| s.start == 0 && s.stop == *l) {
         None
     } else {
@@ -271,7 +270,6 @@ pub fn index_einsum_to_scatter(node: &Index) -> Option<CircuitRc> {
                 .iter()
                 .map(|x| x.unwrap().stop - x.unwrap().start)
                 .collect();
-            dbg!(&new_shape);
             let new_scatter = Scatter::try_new(
                 new_einsum,
                 TensorIndex(
