@@ -93,7 +93,14 @@ impl Schedule {
         result.constants = result
             .constants
             .into_iter()
-            .map(|(key, tensor)| (key, pycall!(f, (key, tensor))))
+            .map(|(key, node)| match node {
+                IrreducibleNode::ArrayConstant(ac) => (
+                    key,
+                    ArrayConstant::new(pycall!(f, (key, ac.value.clone())), ac.name_cloned())
+                        .into(),
+                ),
+                _ => (key, node.clone()),
+            })
             .collect();
         Ok(result)
     }
