@@ -5,8 +5,8 @@ use crate::{
 use macro_rules_attribute::apply;
 use once_cell::sync::Lazy;
 
+use std::fmt::Debug;
 use std::hash::Hash;
-
 pub type EinInt = u8;
 
 /// letters used in einsum and rearrange strings
@@ -222,7 +222,7 @@ pub fn mapping_until_end<T: Eq + Hash + Clone>(x: &T, mapping: &HashMap<T, T>) -
     panic!("mapping_until_end didnt finish");
 }
 
-pub fn apply_fn_until_same<T: Eq + Hash + Clone, F>(x: &T, f: F) -> T
+pub fn apply_fn_until_same<T: Eq + Hash + Clone + Debug, F>(x: &T, f: F) -> T
 where
     F: FnMut(&T) -> T,
 {
@@ -235,13 +235,14 @@ where
         }
         result = next;
         if i == 1000 - 1 {
+            dbg!(&result);
             panic!("apply until same didnt finish");
         }
     }
     result
 }
 
-pub fn apply_fn_until_none<T: Clone, F>(x: &T, f: F) -> T
+pub fn apply_fn_until_none<T: Clone + Debug, F>(x: &T, f: F) -> T
 where
     F: FnMut(&T) -> Option<T>,
 {
@@ -253,7 +254,8 @@ where
             Some(new) => result = new,
         }
         if i == 1000 - 1 {
-            panic!("apply until same didnt finish");
+            dbg!(&result);
+            panic!("apply until none didnt finish");
         }
     }
     result
